clear p 

myModel=inlineModel;
%{
[LONGITUDINAL]
input = {V, k}
EQUATION:
D=100
Cc = D/V*exp(-k*t)

[INDIVIDUAL]
input = {V_pop, omega_V, k_pop, omega_k}
DEFINITION:
V = {distribution=logNormal, reference=V_pop, sd=omega_V}
k = {distribution=logNormal, reference=k_pop, sd=omega_k}

[POPULATION]
input = {Vs, gV, ks, gk}
DEFINITION:
V_pop = {distribution=normal, mean=Vs, sd=gV}
k_pop = {distribution=normal, mean=ks, sd=gk}
%}

adm = struct('time',0,'amount',100);
p = struct('name',{{'Vs', 'gV', 'ks', 'gk', 'omega_V', 'omega_k'}},...
            'value',[10, 0.5, 0.1, 0.01, 0.3, 0.3]);
Cc  = struct('name','Cc','time',0:3:30);
g   = struct('size',1000,'level','individual','output',Cc,'treatment',adm);


p0=p;
p0.value([2 4])=0;
res = simulx('model',myModel,'parameter',p0,'group',g);

q=[90 50 10]';
 f=cell2mat(res{1}.value');
 prct=prctile_mlx(f,q)';
  t=res{1}.time{1}';
 figure(1)
 clf
 subplot(121)
 pl=plot(t,prct);
 set(pl(2),'color',[1 0 0])
 set(pl(3),'color',[0 0.5 0])
 legend({'90%','50%','10%'})
axis([0 30 0 17])
title('10%, 50% and 90% order quantiles')
xlabel('time (h)')
ylabel('concentration (mg/l)')

M=50;
nt=length(t);
nq=length(q);
P=zeros(nt,nq,M);
[res,dataIn] = simulx('model',myModel,'parameter',p,'group',g);
s = struct('loadDesign',false);
for m=1:M
    res = simulx('data',dataIn,'settings',s);
     f=cell2mat(res{1}.VALUE');
    P(:,:,m)=prctile_mlx(f,[90 50 10]')';
end

vt=[t;t(end:-1:1)];
C=[122 197 205 ; 238 130 98 ; 188 238 104]/255;
 subplot(122)
for j=1:3
    pj=squeeze(P(:,j,:));
    pij=prctile_mlx(pj',[5 95])';
    vij=[pij(:,1);pij(end:-1:1,2)];
    fill(vt,vij,C(j,:),'edgecolor',C(j,:))
    hold on
end
axis([0 30 0 17])
title('90% confidence intervals')
xlabel('time (h)')
