function kmplotmlx(res)

%  kmplotmlx.m is governed by the CeCILL-B license. 
%  You can  use, modify and/ or redistribute the software under the terms of 
%  the CeCILL license as circulated by CEA, CNRS and INRIA at the following URL
%  http://www.cecill.info/index.en.html
%
%  kmplotmlx.m was developed by Marc Lavielle (Inria), June 2014. 

clf
N  = length(res{1}.value);
t  = [];
y  = [];
id = [];
for i=1:N
    ri  = res{1}.value{i};
    id  = [id;i*ones(length(ri),1)];
    t   = [t ; res{1}.time{i}(:)];
    y   = [y ; res{1}.value{i}(:)];
end
%% Kaplan Meier (first event)
ij = find(t>0 & y>0);
[temp,ij1,kk] = unique(id(ij),'first');
if length(ij1)<length(kk)
    info.repeated = 1;
else
    info.repeated = 0;
end
[t1,is] = sort(t(ij(ij1)));
ux      = unique(t1);
nt      = cumsum(histc(t1,ux));
p       = [1 ; (N-nt(1:end))/N];
if length(ux)>1
    x = [0 ; kron(ux(1:end-1),ones(2,1)) ; ux(end)];
else
    x = [0 ; ux(end)];
end
p = kron(p,ones(2,1));
x = [x ; ux(end) ; max(t)];

%% Mean number of events
N   = max(id);
nt  = 100;
ij1 = find(y>0 & t>0);
id1 = id(ij1);
t1  = t(ij1);
vt  = linspace(0,max(t),nt);
St  = 0;
for i=1:N
    ii1 = find(id1==i);
    ti1 = t1(ii1);
    ni1 = length(ii1);
    Si  = zeros(1,nt);
    for k=1:ni1
        Si = Si+(vt>=ti1(k));
    end
    St = St+Si;
end
mt = St/N;
vt = vt(:);
mt = mt(:);
if length(vt)>1
    vt = [0 ; kron(vt(1:end-1),ones(2,1)) ; vt(end)];
else
    vt = [0 ; vt(end)];
end
mt = kron(mt,ones(2,1));



%%
X{1} = x(:);
Y{1} = p(:);
X{2} = vt(:);
Y{2} = mt(:);

kmplot(X,Y,info)

function kmplot(X,Y,opt)
tm  = max([cell2mat(X(:,1)) ; cell2mat(X(:,2))]);
nm  = round(max(cell2mat(Y(:,2)))*10)/10 +0.1;
if opt.repeated==1
    subplot(2,1,1)
end
plot(X{1},Y{1})
v = axis;
v(2:4) = [tm 0 1.05];
axis(v)
xlabel('time')
ylabel('P(T_1 > t)')
title('Survival function')
if opt.repeated==1
    subplot(2,1,2)
    plot(X{2},Y{2})
    v = [0 tm -0.02 nm];
    axis(v)
    xlabel('time')
    ylabel('E(N(t))')
    title('Mean number of events per individual')
end

