function  mlxplore(varargin)

%  mlxplore.m is governed by the CeCILL-B license. 
%  You can  use, modify and/ or redistribute the software under the terms of 
%  the CeCILL license as circulated by CEA, CNRS and INRIA at the following URL
%  http://www.cecill.info/index.en.html
%
%  mlxplore.m was developed by Marc Lavielle and Fazia Bellal (Inria), June 2014. 

session = getenv('sessionMlxplore');
setenv('LIXOFT_HOME',session)

tmproject='temp_project.txt';
tmpmodel='temp_model.txt';
%
[model,group,admin,param,output,setting] = readArgs(varargin{:});
%
fid = fopen(tmproject,'w');

%%
fprintf(fid,'%s\n','<MODEL>');
if strcmp(model,tmpmodel)
    ftmp= fopen(tmpmodel,'r');
    tline = fgets(ftmp);
    fprintf(fid,'%s',tline);
   while ischar(tline)
        tline = fgets(ftmp);
        fprintf(fid,'%s',tline);
    end
    fclose(ftmp);
    fprintf(fid,'\n');
else
    fprintf(fid,'%s\n\n',['file=''',model,'''']);
end

%%
if ~isempty(admin)
    fprintf(fid,'%s\n','<DESIGN>');
    fprintf(fid,'%s\n','[ADMINISTRATION]');
    nadmin=length(admin);
    for k=1:nadmin
        admk=admin{k};
        strk2=exstr1(admk);
        strk=['adm',num2str(k),'=',strk2];
        fprintf(fid,'%s\n',strk);
    end
    fprintf(fid,'\n');
    if nadmin>1
        fprintf(fid,'%s\n','[TREATMENT]');
        strt='trt={';
        for k=1:nadmin
            strt=[strt,'adm',num2str(k)];
            if k<nadmin
                strt=[strt,','];
            end
        end
        strt=[strt,'}'];
        fprintf(fid,'%s\n',strt);
    end
    fprintf(fid,'\n');
end

%%
if ~isempty(group)
    fprintf(fid,'%s\n','<DESIGN>');
    fprintf(fid,'%s\n','[ADMINISTRATION]');
    ng=length(group);
    ik=0;
    for i=1:ng
        admin=group{i}.treatment;
        nadmin=length(admin);
        for k=1:nadmin
            ik=ik+1;
            admk=admin{k};
            strk2=exstr1(admk);
            strk=['adm',num2str(ik),'=',strk2];
            fprintf(fid,'%s\n',strk);
        end
    end
    fprintf(fid,'\n');
    fprintf(fid,'%s\n','[TREATMENT]');
    ik=0;
    for i=1:ng
        strt=['trt',num2str(i),'={'];
        admin=group{i}.treatment;
        nadmin=length(admin);
        for k=1:nadmin
            ik=ik+1;
            strt=[strt,'adm',num2str(ik)];
            if k<nadmin
                strt=[strt,','];
            end
        end
        strt=[strt,'}'];
        fprintf(fid,'%s\n',strt);
    end
    fprintf(fid,'\n');
end

%%
fprintf(fid,'%s\n','<PARAMETER>');
param=param{1};
for k=1:length(param.name)
    fprintf(fid,'%s\n',[param.name{k},'=',num2str(param.value(k))]);
end
fprintf(fid,'\n');
%
fprintf(fid,'%s\n','<OUTPUT>');
stro=exstr3(output{1}.name);
fprintf(fid,'%s\n',['list=',stro]);
grid=output{1}.time;
strg=[num2str(grid(1)),':',num2str(grid(2)-grid(1)),':',num2str(grid(end))];
fprintf(fid,'%s\n',['grid=',strg]);
fprintf(fid,'\n');

%
fclose(fid);
%%
system(['"' session '/lib/mlxPlore' '"' ' --project=' tmproject '&'] );

function s=exstr1(a)
fna=fieldnames(a);
s='{';
n=length(fna);
for j=1:n
    eval(['fj=a.',fna{j},';'])
    s=[s,fna{j},'=',exstr2(fj)];
    if j<n
        s=[s,','];
    end
end
s=[s,'}'];

function s=exstr2(a)
if isnumeric(a)
    s='{';
    n=length(a);
    for j=1:n
        s=[s,num2str(a(j))];
        if j<n
            s=[s,','];
        end
    end
    s=[s,'}'];
else
    s=a;
end

function s=exstr3(a)
s='{';
if ~iscell(a)
    a={a};
end
n=length(a);
for j=1:n
    s=[s,num2str(a{j})];
    if j<n
        s=[s,','];
    end
end
s=[s,'}'];


function [model,group,admin,param,output,setting] = readArgs(varargin)


%% Treatment of input arguments
% number of input arguments
nbArgs = length(varargin);
% preallocation
model     = '';
param     = cell(1,0);
output    = cell(1,0);
admin     = cell(1,0);
group     = cell(1,0);
setting   = cell(1,0);

for iInput = 1:2:nbArgs
    if strcmpi(varargin{iInput}, 'treatment')
        admin = tocell(varargin{iInput+1});
    end
    if strcmpi(varargin{iInput}, 'model')
        model = varargin{iInput+1};
        
    end
    if strcmpi(varargin{iInput}, 'group')
        group = tocell(varargin{iInput+1});
        for i=1:length(group)
            group{i}.treatment=tocell(group{i}.treatment);
        end
        if isfield(group{1},'parameter')
            param=tocell(group{1}.parameter);
        end
        if isfield(group{1},'output')
            output=tocell(group{1}.output);
        end
    end
    if strcmpi(varargin{iInput}, 'parameter')
        param = tocell(varargin{iInput+1});
    end
    if strcmpi(varargin{iInput}, 'output')
        output = tocell(varargin{iInput+1});
    end
    if strcmpi(varargin{iInput}, 'setting')
        output = tocell(varargin{iInput+1});
    end
end

% check model
if ~ischar(model)
    error('model must be a string');
end
if strcmp(model,'pkmodel')
    modelFile='temp_model.txt';
    nbparam=length(param{1}.name);
    str1=param{1}.name{1};
    for k=2:nbparam
        str1=[str1, ', ',param{1}.name{k}];
    end
    nbout=length(output{1}.name);
    if nbout==1
        str2=output{1}.name{1};
    else
        str2=['{',output{1}.name{1},',',output{1}.name{2},'}'];
    end
    fid = fopen(modelFile,'w');
    fprintf(fid,'%s\n','[LONGITUDINAL]');
    fprintf(fid,'%s\n',['input={',str1,'}']);
    fprintf(fid,'%s\n','EQUATION:');
    fprintf(fid,'%s\n',[str2,'=pkmodel(',str1,')']);
    fclose(fid);
    model=modelFile;
end

[mpath,mfile,mext]=fileparts(model);
if strcmp(mext,'.xml')
    model=pharmml2mlxtran(model);
end

function c=tocell(s)
% renvoie une cell c a partir de s
if ~iscell(s)
    c{1} = s;
else
    c=s;
end


