function plotmlx(res,varargin)

%  plotmlx.m is governed by the CeCILL-B license. 
%  You can  use, modify and/ or redistribute the software under the terms of 
%  the CeCILL license as circulated by CEA, CNRS and INRIA at the following URL
%  http://www.cecill.info/index.en.html
%
%  plotmlx.m was developed by Marc Lavielle (Inria), June 2014. 

s0=struct;
s0.markersize=get(0,'defaultlinemarkersize');
s0.linewidth=get(0,'defaultlinelinewidth');
s0.fontsize=get(0,'defaultAxesFontSize');

N=length(res{1}.value);

im=[];
M0=length(res);
for m=1:M0
    if isfield(res{m},'time') && ~isempty(cell2mat(res{m}.time))
        im=[im m];
    end
end

io=find(strcmp(varargin,'option'));
if io
    opt=varargin{io+1};
else
    opt=struct;
end
[opt, group] = completeopt(opt,res,im);
res=res(im);
G = length(unique(group));

is=find(strcmp(varargin,'setting'));
if is
    stg=varargin{is+1};
else
    stg=struct;
end

%%
K=length(opt.output);
Nout=zeros(1,K);
%N=cell(1,K);
Tname=cell(1,K);
%Hplot=cell(G,K);
for k=1:K
    lk=opt.output{k};
    %        sk=opt.symbol{k};
    nk=length(lk);
    tk=cell(1,nk);
    %        Nk=cell(1,nk);
    for j=1:nk
        %            Nk{j}=length(lk{j}.value);
        tk{j}=res{lk(j)}.name;
    end
    Nout(k)=nk;
    Tname{k}=tk;
    %        N{k}=Nk;
end

if opt.isplit
    ncolor=max(Nout);
else
    if opt.igroup
        ncolor=hist(group,1:max(group));
    else
        if N==1
            ncolor=max(Nout);
        else
            ncolor=N;
        end
    end
end
stg = completesettings(stg,s0,ncolor);

%%
fz=stg.labelfontsize;
ft=stg.titlefontsize;

hgcf = zeros(1,length(opt.figure));
for j=1:length(opt.figure)
    hgcf(j)=figure(opt.figure(j));
    clf;
end

set(hgcf,'defaultlinemarkersize',stg.markersize);
set(hgcf,'defaultlinelinewidth',stg.linewidth);
set(hgcf,'defaultAxesFontSize',stg.tickfontsize);
set(hgcf,'defaultAxesTickDir','out');

if opt.igroup
    %     if opt.itile
    %         k1=K;
    %         k2=G;
    %     else
    %         k1=G;
    %         k2=K;
    %     end
    
    sp=zeros(1,K*G);
    kg=0;
    xmax=-inf;
    xmin= inf;
    ymin=zeros(1,K);
    ymax=zeros(1,K);
    for k=1:K
        lk=opt.output{k};
        sk=opt.symbol{k};
        nk=length(lk);
        tk=cell(1,nk);
        hk=zeros(1,nk);
        %     ng2=0;
        ymink = inf;
        ymaxk = -inf;
        for g=1:G
            %             ng1=ng2+1;
            %             ng2=ng2+NG(g);
            if opt.itile
                kg=(g-1)*K+k;
                sp(kg)=subplot(G,K,kg);
            else
                kg=kg+1;
                sp(kg)=subplot(K,G,kg);
            end
            set(gca,'ColorOrder',stg.colormap{g})
            hold all
            ig = find(group==g);
            ng=length(ig);
            for j=1:nk
                rj=res{lk(j)};
                sj=sk{j};
                tj=cell2mat(rj.time);
                xmax=max(xmax, max(tj));
                xmin=min(xmin, min(tj));
                i1=0;
                for i=ig
                    i1=i1+1;
                    hj = plot(rj.time{i},rj.value{i},sj);
                    ymink=min(ymink,min(rj.value{i}));
                    ymaxk=max(ymaxk,max(rj.value{i}));
                    %                   end
                    if i1==1
                        hk(j)=hj;
                    end
                end
                tk{j}=rj.name;
            end
            hold off
            if G>1
                title([opt.split,' = ',num2str(g)],'fontsize',ft)
            end
            if  nk>1 && opt.ilegend
                legend(hk,tk,'location','best')
            end
            if nk==1
                ylabel(tk,'fontsize',fz)
                if  opt.ilegend && ng>1
                    leg=cell(1,ng);
                    for i=1:ng
                        leg{i}=num2str(ig(i));
                    end
                    legend(leg,'location','best');
                end
            end
            ymin(k)=ymink;
            ymax(k)=ymaxk;
        end
    end
    
    for k=1:K
        dx=(xmax-xmin)/50;
        dy=(ymax(k)-ymin(k))/50;
        vkg=[xmin-dx, xmax+dx, ymin(k)-dy, ymax(k)+dy];
        for g=1:G
            if opt.itile
                kg=(g-1)*K+k;
                if g==G
                    xlabel(sp(kg),opt.xlabel,'fontsize',fz)
                end
            else
                kg=(k-1)*G+g;
                if k==K
                    xlabel(sp(kg),opt.xlabel,'fontsize',fz)
                end
            end
            axis(sp(kg),vkg)
        end
    end
    
else
    set(hgcf,'DefaultAxesColorOrder',stg.colormap{1});
    if opt.isplit==0
        if K>3
            k2=fix(sqrt(K));
        else
            k2=1;
        end
        k1=ceil(K/k2);
        nplot=K;
    else
        N=length(res{1}.value);
        k2=fix(sqrt(N));
        k1=ceil(N/k2);
        nplot=N;
    end
    if opt.itile
        k0=k1;
        k1=k2;
        k2=k0;
    end
    
    xmax=-inf;
    xmin= inf;
    ymax=-inf(1,K);
    ymin=inf(1,K);
    if opt.isplit==0
        figure(hgcf(1))
    end
    for k=1:K
        if opt.isplit
            figure(hgcf(k))
        else
            subplot(k1,k2,k)
            hold all
        end
        lk=opt.output{k};
        sk=opt.symbol{k};
        nk=length(lk);
        tk=cell(1,nk);
        hk=zeros(1,nk);
        for j=1:nk
            rj=res{lk(j)};
            sj=sk{j};
            nj=length(rj.value);
            tj=cell2mat(rj.time);
            xmax=max(xmax, max(tj));
            xmin=min(xmin, min(tj));
            vj=cell2mat(rj.value);
            ymax(k)=max(ymax(k), max(vj));
            ymin(k)=min(ymin(k), min(vj));
            for i=1:nj
                if ~isempty(rj.time{i})
                    if opt.isplit
                        subplot(k1,k2,i)
                        title(num2str(i),'fontsize',ft)
                        hold all
                    end
                    hj = plot(rj.time{i},rj.value{i},sj);
                    %                    eval(['hj=plot(rj.time{i},rj.value{i},',sj,');'])
                    if i==1
                        hk(j)=hj;
                    end
                end
            end
            %           hk(j)=hj;
            tk{j}=rj.name;
        end
        hold off
        if nk==1
            if opt.isplit==1
                for l=1:k1
                    subplot(k1,k2,k2*(l-1)+1)
                    ylabel(tk,'fontsize',fz)
                end
            else
                ylabel(tk,'fontsize',fz)
                if k==1 && opt.ilegend && nj>1
                    c=get(gca,'children');
                    uc=unique(cell2mat(get(c,'color')),'rows');
                    if (size(uc,1)>1)
                        leg=cell(1,nj);
                        for i=1:nj
                            leg{i}=num2str(i);
                        end
                        legend(leg,'location','best');
                    end
                end
            end
        else
            if opt.ilegend && length(tk)>1
                %       subplot(k1,k2,1)
                legend(hk,tk,'location','best')
            end
        end
        %     end
    end
    for h=1:length(opt.figure)
        figure(opt.figure(h))
        dx=(xmax-xmin)/50;
        dxlim =[xmin-dx,xmax+dx];
        dyh=(ymax(h)-ymin(h))/50;
        dylimh = [ymin(h)-dyh,ymax(h)+dyh];
        for k=1:nplot
            subplot(k1,k2,k)
            xlim(dxlim)
            if opt.isplit
                ylim(dylimh)
            else
                dyk=(ymax(k)-ymin(k))/50;
                dylimk = [ymin(k)-dyk,ymax(k)+dyk];
                ylim(dylimk)
            end
            if k>(k1-1)*k2
                xlabel(opt.xlabel,'fontsize',fz)
            end
        end
    end
    
end

%%
function [opt,group] = completeopt(opt,res,im)
M=length(im);
N=length(res{1}.value);

if ~isfield(opt,'output')
    opt.output=cell(1,M);
    for m=1:M
        opt.output{m}={res{im(m)}.name};
    end
end
if ~iscell(opt.output)
    opt.output={opt.output};
end

nout=length(opt.output);
for m=1:nout
    if ~iscell(opt.output{m})
        opt.output{m}={opt.output{m}};
    end
end

if ~isfield(opt,'symbol')
    opt.symbol='-';
end
if ischar(opt.symbol)
    os=opt.symbol;
    opt.symbol=cell(1,nout);
    for m=1:nout
        nm=length(opt.output{m});
        opt.symbol{m}=cell(1,nm);
        opt.symbol{m}(1:nm)={os};
    end
end
nsymb=length(opt.symbol);
for m=1:nsymb
    if ~iscell(opt.symbol{m})
        opt.symbol{m}={opt.symbol{m}};
    end
end

opt.igroup=0;
opt.isplit=0;
group=ones(N,1);
if isfield(opt,'split')
    if strcmp(opt.split,'individual')
        opt.isplit=1;
    else
        for m=1:length(res)
            if strcmp(res{m}.name,opt.split)
                group=cell2mat(res{m}.value);
            end
        end
    end
end
G=length(unique(group));
if G>1;
    opt.igroup=1;
end

if ~isfield(opt,'tile')
    opt.tile='horizontal';
    if G>1 && nout==1
            opt.tile='vertical';
    end
end
if strcmp(opt.tile,'horizontal')
    opt.itile=1;
else
    opt.itile=0;
end

if ~isfield(opt,'xlabel')
    opt.xlabel='time';
end

opt.ilegend=1;
if isfield(opt,'legend') && strcmp(opt.legend,'off')
    opt.ilegend=0;
end

if ~isfield(opt,'figure')
    if opt.isplit
        opt.figure=1:nout;
    else
        opt.figure=1;
    end
else
    if opt.isplit
        opt.figure=opt.figure(1:nout);
    else
        opt.figure=opt.figure(1);
    end
    
end


opt=findstrcell(opt,res(im));

%%
function s1 = completesettings(s,s0,ncolor)
ns=fieldnames(s);
s1=s;
for j=1:length(ns)
    fj=getfield(s,ns{j});
    setfield(s1,lower(ns{j}),fj);
end

if ~isfield(s1,'markersize')
    s1.markersize=s0.markersize;
end

if ~isfield(s1,'linewidth')
    s1.linewidth=s0.linewidth;
end

if ~isfield(s1,'labelfontsize')
    %    s1.labelfontsize=s0.fontsize;
    s1.labelfontsize=11;
end

if ~isfield(s1,'tickfontsize')
    %    s1.tickfontsize=s0.fontsize;
    s1.tickfontsize=9;
end

if ~isfield(s1,'titlefontsize')
    %    s1.titlefontsize=s0.fontsize;
    s1.titlefontsize=12;
end

nc=length(ncolor);
smap=cell(1,nc);
for j=1:nc
    if ~isfield(s1,'colormap')
        smap{j}=linspecer(ncolor(j));
    else
        eval(['smap{',num2str(j),'}=',s.colormap,'(ncolor(',num2str(j),'));']);
    end
end
s1.colormap=smap;
%%
function o2=findstrcell(o1,r)
lname=cell(1,length(r));
for k=1:length(r)
    lname{k}=r{k}.name;
end
out1=o1.output;
n=length(out1);
for k=1:n
    if ~iscell(out1{k})
        out1{k}={out1{k}};
    end
end
out2=out1;
for k=1:n
    ok1=out1{k};
    nk=length(ok1);
    ok2=ok1;
    uk=[];
    for j=1:nk
        okj=ok1{j};
        if isnumeric(okj)
            ikj=okj;
        else
            ikj=find(strcmp(okj,lname));
        end
        ok2{j}=ikj;
        if ~isempty(ikj)
            uk=[uk j];
        end
    end
    out2{k}=ok2(uk);
    out2{k}=cell2mat(out2{k});
end
o2=o1;
o2.output=out2;

%%
function  s=ptocell(r)
fn=fieldnames(r);
s=r;
for j=1:length(fn)
    nj=fn{j};
    eval(['if ~iscell(r.',nj,') ; s.',nj,'={r.',nj,'}; end;'])
end



