function [dataOut,dataIn] = simulx(varargin)

%  simulx.m is governed by the CeCILL-B license. 
%  You can  use, modify and/ or redistribute the software under the terms of 
%  the CeCILL license as circulated by CEA, CNRS and INRIA at the following URL
%  http://www.cecill.info/index.en.html
%
%  simulx.m was developed by Marc Lavielle and Fazia Bellal (Inria), June 2014. 

session = getenv('sessionSimulx');
setenv('LIXOFT_HOME',session)

if strcmp(varargin{1},'data')
    dataIn = varargin{2};
    handleSimulx = @mlxCompute;
    if nargin>=4 && strcmp(varargin{3},'settings')
        s = processing_setting(varargin{4});
        dataOut       = handleSimulx('DATA', dataIn, 'SETTINGS', s);
    else
        dataOut       = handleSimulx('DATA', dataIn);
    end
    %     dmodel        = dataIn.model;
    %     dsystem       = dataIn.system;
    %     dataIn        = rmfield(dataIn,{'model','system'});
else
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Initialization
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    dataIn       = struct();
    nbIndsParam  = 0;
    nbIndsAdmin  = 0;
    nbIndsOutput = 0;
    lixoftSystem.mexunlock=false;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Read arguments
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [model,group,admin,param,output,settings,project] = readArgs(varargin{:});
    
    settings = processing_setting(settings);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Monolix project
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if ~isempty(project)
        mlxtranfile                   = project;
        project                       = struct('mlxtranfile',mlxtranfile, 'session', session );
        [model, admin, param, output] = processing_monolix(project,model, admin, param, output );
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Manage parameters
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if ~isempty(param)
        [paramInd, paramPop, nbIndsParam]      = processing_parameter(param);
        if ~isempty(paramPop)
            dataIn.population_parameters.name  = paramPop.name;
            dataIn.population_parameters.value = paramPop.value;
        end
    else
        paramInd = {};
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Manage sources
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if ~isempty(admin)
        [admin, nbIndsAdmin] = processing_administration(admin);
    else
        admin = {};
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Manage outputs
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    outputName=listname(output);
    if ~isempty(output)
        [outputIndividual, outputGroup, nbIndsOutput] = processing_output(output);
        output                                        = struct();
        output.individual                             = outputIndividual;
        output.group                                  = outputGroup;
    else
        output = {};
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Manage groups
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    nb_group = max([nbIndsParam nbIndsAdmin nbIndsOutput]);
    group    = processing_group(group,admin, output,paramInd,nb_group );
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Manage targets
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [depot, group] = processing_target(group);
    if ~isempty(depot)
        dataIn.depot = depot;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Manage data
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    dataIn = processing_data(dataIn,group);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Call mlxCompute MATLAB connector
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    dataIn.MODEL=model;
    handleSimulx = @mlxCompute;
    if ~isempty(settings)
        datah = handleSimulx('DATA', dataIn,'SETTINGS',settings);
    else
        datah = handleSimulx('DATA', dataIn);
        %        datah = handleSimulx('MODEL', model,'DATA', dataIn, 'system', lixoftSystem);
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Formating of simulx output
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    n       = length(datah);
    dataOut = cell(1,n);
    for k = 1 : n
        oldField = fieldnames(datah{k});
        newField = lower(oldField);
        temp     = struct;
        for ii = 1:length(oldField)
            temp.(newField{ii}) = datah{k}.(oldField{ii});
        end
        ks = strcmp(outputName,temp.name);
        if ~isempty(ks)
            dataOut{ks} = temp;
        else
            dataOut{k}  = temp;
        end
    end
    %     dataIn.model  = model;
    %     dataIn.system = lixoftSystem;
end
if exist('group','var')
    N=length(dataOut{1}.value);
    g=cell(1,N);
    K=length(group);
    if K>1
        i2=0;
        for k=1:K
            i1=i2+1;
            i2=i2+group{k}.size;
            g(i1:i2)={k};
        end
        dataOut{n+1}.name = 'group';
        dataOut{n+1}.value=g;
    end
end

