function modelFile = inlineModel(varargin)
%INLINEMODEL  Get the text that appears in the next comment block.
%
%Example,
%
%   model = inlineModel;
%   %{
%   [LONGITUDINAL]
%   input = {A, k}
%   EQUATION:
%   C = A*exp(-k*t)
%   %}
%
% model is a cell array of strings

if nargin==1
    modelFile=varargin{1};
else
        modelFile='simulx_model.txt';
end
[dbs,thisWorkspace] = dbstack('-completenames');
assert(length(dbs) > 1,'inlineModel must be called from an M-file.')
dbs = dbs(thisWorkspace + 1);

fid = fopen(dbs.file);
try
    strl='';
	textscan(fid,'%*s',0,'HeaderLines',dbs.line,'Delimiter','');
	while ~isequal(strl,-1) && isempty(regexp(strl,'^\s*%{','once'))
		strl = fgetl(fid);
	end
	k = 0;
	while true
		k = k + 1;
		str{k} = fgetl(fid);
		if isequal(str{k},-1) || ...
				~isempty(regexp(str{k},'^\s*%}','once'))
			break
		end
	end
	fclose(fid);
catch err
	fclose(fid);
	rethrow(err)
end
str=str(1:k-1);

fid = fopen(modelFile,'w');
fprintf(fid,'%s\n',str{:});
fclose(fid);

