function datas = readdatamlx(infoProject)
% READDATAMLX
%
% READDATAMLX reads a datafile and create a cell-array.
%
%
% This software is governed by the CeCILL-B license. You can  use,
% modify and/ or redistribute the software under the terms of the CeCILL
% license as circulated by CEA, CNRS and INRIA at the following URL
% "http://www.cecill.info/index.en.html".

% Marc Lavielle, Inria
% May 2013

header          = infoProject.dataheader;
observationName = infoProject.output;
headerList      = {'ID','TIME','AMT','ADM','RATE','TINF','Y','YTYPE','X','COV','CAT'};
 newList={'id','time','amount','type','rate','tinf','y','ytype','x','cov','cat'};
% newList         = cellfun(@lower,headerList,'UniformOutput',false);
newHeader       = cell(1,length(header));
nlabel=length(header);

for i=1:length(headerList)
    hi=headerList{i};
    eval(['ii=''i',lower(hi),''';'])
    eval([ii,'=find(strcmp(header,''',hi,'''));'])
    eval(['ih=',ii,';'])
    if ~isempty(ih)
        newHeader(ih)=newList(i);
    end;
end


fileFormat='';
if isstruct(infoProject)
    if isfield(infoProject, 'dataformat')
        fileFormat=infoProject.dataformat;
    end
    datafile=infoProject.datafile;
end
if isempty(fileFormat)
    [f,p,e]=fileparts(datafile);
    if strcmp(e,'.csv')
        fileFormat='csv';
    else
%        fileFormat='\t';
        fileFormat=0;
    end
end
switch (lower(fileFormat))
    case {'csv'}
        fid = fopen(datafile);
        h1 = textscan(fid,'%s',nlabel,'delimiter',',');
        fclose(fid);
        fid = fopen(datafile);
        h2 = textscan(fid,'%s',nlabel,'delimiter',';');
        fclose(fid);
        if length(h1{1}{1})>length(h2{1}{1})
            delimiter=';';
        else
            delimiter=',';
        end
    case {'space'}
        delimiter=' ';
    case {'tab'}
        delimiter='\t';
    case {';'}
        delimiter=';';
    case {'\t'}     % rajout Fazia   
        delimiter='\t'; 
    case {0}        
        delimiter=0; 
    otherwise
        delimiter=',';
end

%%
fid = fopen(datafile);
if delimiter == 0
S0 = textscan(fid,'%s',nlabel);
else
S0 = textscan(fid,'%s',nlabel,'delimiter',delimiter);
end
S0=S0{1};
newHeader(icov)=S0(icov);
newHeader(icat)=S0(icat);

fs=repmat('%s ',1,nlabel);
fs(3*(itime-1)+2)='f';
fs(3*(ix-1)+2)='f';
fs(3*(icov-1)+2)='f';
if delimiter == 0
S = textscan(fid,fs);
else
S = textscan(fid,fs,'delimiter',delimiter);
end
fclose(fid);

%%
ls=zeros(1,nlabel);
for k=1:nlabel
    ls(k)=length(S{k});
end
lmin=min(ls);
for k=1:nlabel
    S{k}=S{k}(1:lmin);
end

[iduf,iuf,idnumf]  = unique(S{iid},'first');
[ia,ib]            = sort(iuf);
[~,ic]             = sort(ib);
ids                = iduf(ib);
iu                 = ia;
idnum              = ic(idnumf);
N                  = length(iduf);

iop_id = 0;
i      = 1;
while iop_id==0 && i<=N
    iop_id= strcmp(ids{i},num2str(i));
    i=i+1;
end

if isempty(itime)
    itime = ix(1);
    ix    = ix(2:end);
end
t  = S{itime};
nx = length(ix);


%%************************************************************************
%       SOURCE FIELD
%**************************************************************************
if ~isempty(iamt)
    ixdose=[ iamt irate iadm];
else
    ixdose=zeros(1,0);
end
if ~isempty(ixdose)
    i1      = find(~strcmp(S{ixdose(1)},'.'));
    ndose   = length(i1);
    nxdose  = length(ixdose);
    u.label = 'source';
    u.name  = 'doseRegimen';
    u.header= {'id',newHeader{itime},newHeader{ixdose}};
    uv      = [idnum(i1) t(i1) zeros(ndose,length(ixdose))];
    
    for i=1:ndose
        for j=1:nxdose
            uv(i,2+j)=str2double(S{ixdose(j)}(i1(i)));
        end
    end
    u.value       = uv;
    u             = {u};
    datas.source = u;
end
%%************************************************************************
%       OBSERVATION FIELD
%**************************************************************************
iobs1=find(~strcmp(S{iy},'.'));
nobs=length(iobs1);
yvalues=[idnum(iobs1) t(iobs1) zeros(nobs,nx)] ;
for j=1:nx
        yvalues(:,2+j)=S{ix(j)}(iobs1);
end
if ~isempty(iytype)
    ytype=str2double(S{iytype}(iobs1));
    nyu=min(length(observationName),length(unique(ytype)));
else
    ytype=ones(nobs,1);
    nyu=1;
end

observation=struct('name',observationName);
y=cell(1,nyu);
for k=1:nyu
    y{k}.label='observation';
    ik=(ytype==k);
    yvk=yvalues(ik,:);
    y{k}.name=observation(k).name;
    y{k}.header={'id',newHeader{itime},newHeader{ix}};
    y{k}.value=yvk;
end
datas.observation=y;

%%************************************************************************
%       COVARIATE FIELD
%**************************************************************************
ncov=length(icov);
ncat=length(icat);
nc=ncov+ncat;
if nc>0
    c=cell(1,nc);
    [~,isc]=sort([icov icat]);
    iscov=isc(1:ncov);
    iscat=isc(ncov+1:nc);
    for k=1:ncov
        ic=iscov(k);
        c{ic}.name=newHeader{icov(k)};
        c{ic}.value=[(1:N)' S{icov(k)}(iu)];
    end
    for k=1:ncat
        ic=iscat(k);
        [~,~,cc]=unique(S{icat(k)}(iu));
        c{ic}.name=newHeader{icat(k)};
        c{ic}.value=[(1:N)' cc];
    end
    for k=1:nc
        c{k}.label='covariate';
        c{k}.header={'id', c{k}.name};
        c{k}=orderfields(c{k},{'name','label','header','value'});
    end
    datas.covariate=c;
end
